@@ -1,7 +1,3 @@
-from __future__ import absolute_import
-from __future__ import unicode_literals
-
-
 def raw_cte_sql(sql, params, refs):
     """Raw CTE SQL
 
@@ -11,14 +7,14 @@ def raw_cte_sql(sql, params, refs):
     :returns: Object that can be passed to `With`.
     """
 
-    class raw_cte_ref(object):
+    class raw_cte_ref:
         def __init__(self, output_field):
             self.output_field = output_field
 
         def get_source_expressions(self):
             return []
 
-    class raw_cte_compiler(object):
+    class raw_cte_compiler:
 
         def __init__(self, connection):
             self.connection = connection
@@ -29,8 +25,8 @@ def raw_cte_sql(sql, params, refs):
         def quote_name_unless_alias(self, name):
             return self.connection.ops.quote_name(name)
 
-    class raw_cte_queryset(object):
-        class query(object):
+    class raw_cte_queryset:
+        class query:
             @staticmethod
             def get_compiler(connection, *, elide_empty=None):
                 return raw_cte_compiler(connection)
