@@ -1,19 +1,14 @@
-from __future__ import absolute_import
-from __future__ import unicode_literals
-
-from __future__ import print_function
-from unittest import SkipTest
-
+import pytest
 from django.db.models import IntegerField, TextField
 from django.db.models.aggregates import Count, Max, Min, Sum
 from django.db.models.expressions import (
     Exists, ExpressionWrapper, F, OuterRef, Subquery,
 )
 from django.db.models.sql.constants import LOUTER
+from django.db.utils import OperationalError, ProgrammingError
 from django.test import TestCase
 
-from django_cte import With
-from django_cte import CTEManager
+from django_cte import CTE, with_cte
 
 from .models import Order, Region, User
 
@@ -24,22 +19,20 @@ text_field = TextField()
 class TestCTE(TestCase):
 
     def test_simple_cte_query(self):
-        cte = With(
+        cte = CTE(
             Order.objects
             .values("region_id")
             .annotate(total=Sum("amount"))
         )
 
-        orders = (
-            # FROM orders INNER JOIN cte ON orders.region_id = cte.region_id
-            cte.join(Order, region=cte.col.region_id)
+        orders = with_cte(
+            # WITH cte ...
+            cte,
 
-            # Add `WITH ...` before `SELECT ... FROM orders ...`
-            .with_cte(cte)
-
-            # Annotate each Order with a "region_total"
-            .annotate(region_total=cte.col.total)
-        )
+            # SELECT ... FROM orders
+            # INNER JOIN cte ON orders.region_id = cte.region_id
+            select=cte.join(Order, region=cte.col.region_id),
+        ).annotate(region_total=cte.col.total)
         print(orders.query)
 
         data = sorted((o.amount, o.region_id, o.region_total) for o in orders)
@@ -69,17 +62,16 @@ class TestCTE(TestCase):
         ])
 
     def test_cte_name_escape(self):
-        totals = With(
+        totals = CTE(
             Order.objects
             .filter(region__parent="sun")
             .values("region_id")
             .annotate(total=Sum("amount")),
             name="mixedCaseCTEName"
         )
-        orders = (
-            totals
-            .join(Order, region=totals.col.region_id)
-            .with_cte(totals)
+        orders = with_cte(
+            totals,
+            select=totals.join(Order, region=totals.col.region_id)
             .annotate(region_total=totals.col.total)
             .order_by("amount")
         )
@@ -87,15 +79,14 @@ class TestCTE(TestCase):
             str(orders.query).startswith('WITH RECURSIVE "mixedCaseCTEName"'))
 
     def test_cte_queryset(self):
-        sub_totals = With(
+        sub_totals = CTE(
             Order.objects
             .values(region_parent=F("region__parent_id"))
             .annotate(total=Sum("amount")),
         )
-        regions = (
-            Region.objects.all()
-            .with_cte(sub_totals)
-            .annotate(
+        regions = with_cte(
+            sub_totals,
+            select=Region.objects.annotate(
                 child_regions_total=Subquery(
                     sub_totals.queryset()
                     .filter(region_parent=OuterRef("name"))
@@ -122,11 +113,14 @@ class TestCTE(TestCase):
         ])
 
     def test_cte_queryset_with_model_result(self):
-        cte = With(
+        cte = CTE(
             Order.objects
             .annotate(region_parent=F("region__parent_id")),
         )
-        orders = cte.queryset().with_cte(cte)
+        orders = with_cte(
+            cte,         # WITH cte AS (...)
+            select=cte,  # SELECT ... FROM cte
+        )
         print(orders.query)
 
         data = sorted(
@@ -144,13 +138,13 @@ class TestCTE(TestCase):
         )
 
     def test_cte_queryset_with_join(self):
-        cte = With(
+        cte = CTE(
             Order.objects
             .annotate(region_parent=F("region__parent_id")),
         )
-        orders = (
-            cte.queryset()
-            .with_cte(cte)
+        orders = with_cte(
+            cte,
+            select=cte.queryset()
             .annotate(parent=F("region__parent_id"))
             .order_by("region_id", "amount")
         )
@@ -166,7 +160,7 @@ class TestCTE(TestCase):
         ])
 
     def test_cte_queryset_with_values_result(self):
-        cte = With(
+        cte = CTE(
             Order.objects
             .values(
                 "region_id",
@@ -174,11 +168,7 @@ class TestCTE(TestCase):
             )
             .distinct()
         )
-        values = (
-            cte.queryset()
-            .with_cte(cte)
-            .filter(region_parent__isnull=False)
-        )
+        values = with_cte(cte, select=cte).filter(region_parent__isnull=False)
         print(values.query)
 
         def key(item):
@@ -197,27 +187,27 @@ class TestCTE(TestCase):
         ])
 
     def test_named_simple_ctes(self):
-        totals = With(
+        totals = CTE(
             Order.objects
             .filter(region__parent="sun")
             .values("region_id")
             .annotate(total=Sum("amount")),
             name="totals",
         )
-        region_count = With(
+        region_count = CTE(
             Region.objects
             .filter(parent="sun")
-            .values("parent")
+            .values("parent_id")
             .annotate(num=Count("name")),
             name="region_count",
         )
-        orders = (
-            region_count.join(
+        orders = with_cte(
+            totals,
+            region_count,
+            select=region_count.join(
                 totals.join(Order, region=totals.col.region_id),
                 region__parent=region_count.col.parent_id
             )
-            .with_cte(totals)
-            .with_cte(region_count)
             .annotate(region_total=totals.col.total)
             .annotate(region_count=region_count.col.num)
             .order_by("amount")
@@ -261,9 +251,9 @@ class TestCTE(TestCase):
                 ),
                 all=True,
             )
-        rootmap = With.recursive(make_root_mapping, name="rootmap")
+        rootmap = CTE.recursive(make_root_mapping, name="rootmap")
 
-        totals = With(
+        totals = CTE(
             rootmap.join(Order, region_id=rootmap.col.name)
             .values(
                 root=rootmap.col.root,
@@ -274,11 +264,10 @@ class TestCTE(TestCase):
             name="totals",
         )
 
-        root_regions = (
-            totals.join(Region, name=totals.col.root)
-            .with_cte(rootmap)
-            .with_cte(totals)
-            .annotate(
+        root_regions = with_cte(
+            rootmap,
+            totals,
+            select=totals.join(Region, name=totals.col.root).annotate(
                 # count of orders in this region and all subregions
                 orders_count=totals.col.orders_count,
                 # sum of order amounts in this region and all subregions
@@ -296,17 +285,16 @@ class TestCTE(TestCase):
         ])
 
     def test_materialized_option(self):
-        totals = With(
+        totals = CTE(
             Order.objects
             .filter(region__parent="sun")
             .values("region_id")
             .annotate(total=Sum("amount")),
             materialized=True
         )
-        orders = (
-            totals
-            .join(Order, region=totals.col.region_id)
-            .with_cte(totals)
+        orders = with_cte(
+            totals,
+            select=totals.join(Order, region=totals.col.region_id)
             .annotate(region_total=totals.col.total)
             .order_by("amount")
         )
@@ -317,14 +305,14 @@ class TestCTE(TestCase):
         )
 
     def test_update_cte_query(self):
-        cte = With(
+        cte = CTE(
             Order.objects
             .values(region_parent=F("region__parent_id"))
             .annotate(total=Sum("amount"))
             .filter(total__isnull=False)
         )
         # not the most efficient query, but it exercises CTEUpdateQuery
-        Order.objects.all().with_cte(cte).filter(region_id__in=Subquery(
+        with_cte(cte, select=Order).filter(region_id__in=Subquery(
             cte.queryset()
             .filter(region_parent=OuterRef("region_id"))
             .values("region_parent")
@@ -348,7 +336,7 @@ class TestCTE(TestCase):
 
     def test_update_with_subquery(self):
         # Test for issue: https://github.com/dimagi/django-cte/issues/9
-        # Issue is not reproduces on sqlite3 use postgres to run.
+        # Issue is not reproduced on sqlite3, use postgres to run.
         # To reproduce the problem it's required to have some join
         # in the select-query so the compiler will turn it into a subquery.
         # To add a join use a filter over field of related model
@@ -362,19 +350,21 @@ class TestCTE(TestCase):
             ('mars', 0),
         })
 
+    @pytest.mark.xfail(
+        reason="this test will not work until `QuerySet.delete` "
+            "(Django method) calls `self.query.chain(sql.DeleteQuery)` "
+            "instead of `sql.DeleteQuery(self.model)`",
+        raises=(OperationalError, ProgrammingError),
+        strict=True,
+    )
     def test_delete_cte_query(self):
-        raise SkipTest(
-            "this test will not work until `QuerySet.delete` (Django method) "
-            "calls `self.query.chain(sql.DeleteQuery)` instead of "
-            "`sql.DeleteQuery(self.model)`"
-        )
-        cte = With(
+        cte = CTE(
             Order.objects
             .values(region_parent=F("region__parent_id"))
             .annotate(total=Sum("amount"))
             .filter(total__isnull=False)
         )
-        Order.objects.all().with_cte(cte).annotate(
+        with_cte(cte, select=Order).annotate(
             cte_has_order=Exists(
                 cte.queryset()
                 .values("total")
@@ -395,7 +385,7 @@ class TestCTE(TestCase):
     def test_outerref_in_cte_query(self):
         # This query is meant to return the difference between min and max
         # order of each region, through a subquery
-        min_and_max = With(
+        min_and_max = CTE(
             Order.objects
             .filter(region=OuterRef("pk"))
             .values('region')  # This is to force group by region_id
@@ -409,7 +399,8 @@ class TestCTE(TestCase):
             Region.objects
             .annotate(
                 difference=Subquery(
-                    min_and_max.queryset().with_cte(min_and_max).annotate(
+                    with_cte(min_and_max, select=min_and_max)
+                    .annotate(
                         difference=ExpressionWrapper(
                             F('amount_max') - F('amount_min'),
                             output_field=int_field,
@@ -438,16 +429,16 @@ class TestCTE(TestCase):
         ])
 
     def test_experimental_left_outer_join(self):
-        totals = With(
+        totals = CTE(
             Order.objects
             .values("region_id")
             .annotate(total=Sum("amount"))
             .filter(total__gt=100)
         )
-        orders = (
-            totals
+        orders = with_cte(
+            totals,
+            select=totals
             .join(Order, region=totals.col.region_id, _join_type=LOUTER)
-            .with_cte(totals)
             .annotate(region_total=totals.col.total)
         )
         print(orders.query)
@@ -486,9 +477,7 @@ class TestCTE(TestCase):
         subquery model doesn't use the CTE manager, and the query results
         match expected behavior
         """
-        self.assertNotIsInstance(User.objects, CTEManager)
-
-        sub_totals = With(
+        sub_totals = CTE(
             Order.objects
             .values(region_parent=F("region__parent_id"))
             .annotate(
@@ -500,10 +489,9 @@ class TestCTE(TestCase):
                 ),
             ),
         )
-        regions = (
-            Region.objects.all()
-            .with_cte(sub_totals)
-            .annotate(
+        regions = with_cte(
+            sub_totals,
+            select=Region.objects.annotate(
                 child_regions_total=Subquery(
                     sub_totals.queryset()
                     .filter(region_parent=OuterRef("name"))
@@ -535,31 +523,32 @@ class TestCTE(TestCase):
         correct position
         """
 
-        totals = With(
+        totals = CTE(
             Order.objects
             .filter(region__parent="sun")
             .values("region_id")
             .annotate(total=Sum("amount")),
             name="totals",
         )
-        region_count = With(
+        region_count = CTE(
             Region.objects
             .filter(parent="sun")
-            .values("parent")
+            .values("parent_id")
             .annotate(num=Count("name")),
             name="region_count",
         )
-        orders = (
-            region_count.join(
+        orders = with_cte(
+            totals,
+            region_count,
+            select=region_count.join(
                 totals.join(Order, region=totals.col.region_id),
                 region__parent=region_count.col.parent_id
             )
-            .with_cte(totals)
-            .with_cte(region_count)
             .annotate(region_total=totals.col.total)
             .annotate(region_count=region_count.col.num)
             .order_by("amount")
         )
+        print(orders.query)
 
         self.assertIsInstance(orders.explain(), str)
 
@@ -568,16 +557,16 @@ class TestCTE(TestCase):
         Verifies that the CTEQueryCompiler can handle empty result sets in the
         related CTEs
         """
-        totals = With(
+        totals = CTE(
             Order.objects
             .filter(id__in=[])
             .values("region_id")
             .annotate(total=Sum("amount")),
             name="totals",
         )
-        orders = (
-            totals.join(Order, region=totals.col.region_id)
-            .with_cte(totals)
+        orders = with_cte(
+            totals,
+            select=totals.join(Order, region=totals.col.region_id)
             .annotate(region_total=totals.col.total)
             .order_by("amount")
         )
@@ -585,18 +574,97 @@ class TestCTE(TestCase):
         self.assertEqual(len(orders), 0)
 
     def test_left_outer_join_on_empty_result_set_cte(self):
-        totals = With(
+        totals = CTE(
             Order.objects
             .filter(id__in=[])
             .values("region_id")
             .annotate(total=Sum("amount")),
             name="totals",
         )
-        orders = (
-            totals.join(Order, region=totals.col.region_id, _join_type=LOUTER)
-            .with_cte(totals)
+        orders = with_cte(
+            totals,
+            select=totals
+            .join(Order, region=totals.col.region_id, _join_type=LOUTER)
             .annotate(region_total=totals.col.total)
             .order_by("amount")
         )
 
         self.assertEqual(len(orders), 22)
+
+    def test_union_query_with_cte(self):
+        orders = (
+            Order.objects
+            .filter(region__parent="sun")
+            .only("region", "amount")
+        )
+        orders_cte = CTE(orders, name="orders_cte")
+        orders_cte_queryset = orders_cte.queryset()
+
+        earth_orders = orders_cte_queryset.filter(region="earth")
+        mars_orders = orders_cte_queryset.filter(region="mars")
+
+        earth_mars = earth_orders.union(mars_orders, all=True)
+        earth_mars_cte = with_cte(
+            orders_cte,
+            select=earth_mars
+            .order_by("region", "amount")
+            .values_list("region", "amount")
+        )
+        print(earth_mars_cte.query)
+
+        self.assertEqual(list(earth_mars_cte), [
+            ('earth', 30),
+            ('earth', 31),
+            ('earth', 32),
+            ('earth', 33),
+            ('mars', 40),
+            ('mars', 41),
+            ('mars', 42),
+        ])
+
+    def test_cte_select_pk(self):
+        orders = Order.objects.filter(region="earth").values("pk")
+        cte = CTE(orders)
+        queryset = with_cte(
+            cte, select=cte.join(orders, pk=cte.col.pk)
+        ).order_by("pk")
+        print(queryset.query)
+        self.assertEqual(list(queryset), [
+            {'pk': 9},
+            {'pk': 10},
+            {'pk': 11},
+            {'pk': 12},
+        ])
+
+    def test_django52_resolve_ref_regression(self):
+        cte = CTE(
+            Order.objects.annotate(
+                pnt_id=F("region__parent_id"),
+                region_name=F("region__name"),
+            ).values(
+                # important: more than one query.select field
+                "region_id",
+                "amount",
+                # important: more than one query.annotations field
+                "pnt_id",
+                "region_name",
+            )
+        )
+        qs = with_cte(
+            cte,
+            select=cte.queryset()
+            .values(
+                amt=cte.col.amount,
+                pnt_id=cte.col.pnt_id,
+                region_name=cte.col.region_name,
+            )
+            .filter(region_id="earth")
+            .order_by("amount")
+        )
+        print(qs.query)
+        self.assertEqual(list(qs), [
+            {'amt': 30, 'region_name': 'earth', 'pnt_id': 'sun'},
+            {'amt': 31, 'region_name': 'earth', 'pnt_id': 'sun'},
+            {'amt': 32, 'region_name': 'earth', 'pnt_id': 'sun'},
+            {'amt': 33, 'region_name': 'earth', 'pnt_id': 'sun'},
+        ])
