@@ -1,7 +1,3 @@
-from __future__ import absolute_import
-from __future__ import print_function
-from __future__ import unicode_literals
-
 import pickle
 from unittest import SkipTest
 
@@ -20,7 +16,7 @@ from django.db.models.functions import Concat
 from django.db.utils import DatabaseError
 from django.test import TestCase
 
-from django_cte import With
+from django_cte import CTE, with_cte
 
 from .models import KeyPair, Region
 
@@ -52,11 +48,11 @@ class TestRecursiveCTE(TestCase):
                 all=True,
             )
 
-        cte = With.recursive(make_regions_cte)
+        cte = CTE.recursive(make_regions_cte)
 
-        regions = (
-            cte.join(Region, name=cte.col.name)
-            .with_cte(cte)
+        regions = with_cte(
+            cte,
+            select=cte.join(Region, name=cte.col.name)
             .annotate(
                 path=cte.col.path,
                 depth=cte.col.depth,
@@ -112,8 +108,10 @@ class TestRecursiveCTE(TestCase):
                 ),
                 all=True,
             )
-        cte = With.recursive(make_regions_cte)
-        regions = cte.join(Region, name=cte.col.name).with_cte(cte).annotate(
+        cte = CTE.recursive(make_regions_cte)
+        regions = with_cte(
+            cte, select=cte.join(Region, name=cte.col.name)
+        ).annotate(
             path=cte.col.path,
             depth=cte.col.depth,
             is_planet=cte.col.is_planet,
@@ -139,8 +137,8 @@ class TestRecursiveCTE(TestCase):
                 cte.join(Region, parent=cte.col.name),
                 all=True,
             )
-        cte = With.recursive(make_regions_cte)
-        regions = cte.join(Region, name=cte.col.name).with_cte(cte)
+        cte = CTE.recursive(make_regions_cte)
+        regions = with_cte(cte, select=cte.join(Region, name=cte.col.name))
 
         print(regions.query)
         try:
@@ -168,8 +166,8 @@ class TestRecursiveCTE(TestCase):
             return cte.join(Region, parent=cte.col.name).values(
                 depth=cte.col.depth + 1,
             )
-        cte = With.recursive(make_bad_cte)
-        regions = cte.join(Region, name=cte.col.name).with_cte(cte)
+        cte = CTE.recursive(make_bad_cte)
+        regions = with_cte(cte, select=cte.join(Region, name=cte.col.name))
         with self.assertRaises(ValueError) as context:
             print(regions.query)
         self.assertIn("Circular reference:", str(context.exception))
@@ -188,11 +186,10 @@ class TestRecursiveCTE(TestCase):
                 ),
                 all=True,
             )
-        cte = With.recursive(make_regions_cte)
-        regions = (
-            Region.objects.all()
-            .with_cte(cte)
-            .annotate(_ex=Exists(
+        cte = CTE.recursive(make_regions_cte)
+        regions = with_cte(
+            cte,
+            select=Region.objects.annotate(_ex=Exists(
                 cte.queryset()
                 .values(value=Value("1", output_field=int_field))
                 .filter(name=OuterRef("name"))
@@ -217,8 +214,8 @@ class TestRecursiveCTE(TestCase):
                 ),
                 all=True,
             )
-        cte = With.recursive(make_regions_cte)
-        regions = cte.queryset().with_cte(cte).filter(depth=2).order_by("name")
+        cte = CTE.recursive(make_regions_cte)
+        regions = with_cte(cte, select=cte).filter(depth=2).order_by("name")
 
         pickled_qs = pickle.loads(pickle.dumps(regions))
 
@@ -234,30 +231,28 @@ class TestRecursiveCTE(TestCase):
                 value=F('name'),
             ).union(
                 cte.join(
-                    Region.objects.all().annotate(
-                        value=F('name'),
-                    ),
+                    Region.objects.annotate(value=F('name')),
                     parent_id=cte.col.name,
                 ),
                 all=True,
             )
-        cte = With.recursive(make_regions_cte)
-        query = cte.queryset().with_cte(cte)
+        cte = CTE.recursive(make_regions_cte)
+        query = with_cte(cte, select=cte)
 
-        exclude_leaves = With(cte.queryset().filter(
+        exclude_leaves = CTE(cte.queryset().filter(
             parent__name='sun',
         ).annotate(
             value=Concat(F('name'), F('name'))
         ), name='value_cte')
 
-        query = query.annotate(
+        query = with_cte(exclude_leaves, select=query.annotate(
             _exclude_leaves=Exists(
                 exclude_leaves.queryset().filter(
                     name=OuterRef("name"),
                     value=OuterRef("value"),
                 )
             )
-        ).filter(_exclude_leaves=True).with_cte(exclude_leaves)
+        ).filter(_exclude_leaves=True))
         print(query.query)
 
         # Nothing should be returned.
@@ -272,23 +267,23 @@ class TestRecursiveCTE(TestCase):
                 rank=F('value'),
             ).union(
                 cte.join(
-                    KeyPair.objects.all().order_by(),
+                    KeyPair.objects.order_by(),
                     parent_id=cte.col.id,
                 ).annotate(
                     rank=F('value'),
                 ),
                 all=True,
             )
-        cte = With.recursive(make_regions_cte)
-        children = cte.queryset().with_cte(cte)
+        cte = CTE.recursive(make_regions_cte)
+        children = with_cte(cte, select=cte)
 
-        xdups = With(cte.queryset().filter(
+        xdups = CTE(cte.queryset().filter(
             parent__key="level 1",
         ).annotate(
             rank=F('value')
         ).values('id', 'rank'), name='xdups')
 
-        children = children.annotate(
+        children = with_cte(xdups, select=children.annotate(
             _exclude=Exists(
                 (
                     xdups.queryset().filter(
@@ -297,7 +292,7 @@ class TestRecursiveCTE(TestCase):
                     )
                 )
             )
-        ).filter(_exclude=True).with_cte(xdups)
+        ).filter(_exclude=True))
 
         print(children.query)
         query = KeyPair.objects.filter(parent__in=children)
@@ -315,10 +310,25 @@ class TestRecursiveCTE(TestCase):
         # This test covers MATERIALIZED option in SQL query
         def make_regions_cte(cte):
             return KeyPair.objects.all()
-        cte = With.recursive(make_regions_cte, materialized=True)
+        cte = CTE.recursive(make_regions_cte, materialized=True)
 
-        query = KeyPair.objects.with_cte(cte)
+        query = with_cte(cte, select=KeyPair)
         print(query.query)
         self.assertTrue(
             str(query.query).startswith('WITH RECURSIVE "cte" AS MATERIALIZED')
         )
+
+    def test_recursive_self_queryset(self):
+        def make_regions_cte(cte):
+            return Region.objects.filter(
+                pk="earth"
+            ).values("pk").union(
+                cte.join(Region, parent=cte.col.pk).values("pk")
+            )
+        cte = CTE.recursive(make_regions_cte)
+        queryset = with_cte(cte, select=cte).order_by("pk")
+        print(queryset.query)
+        self.assertEqual(list(queryset), [
+            {'pk': 'earth'},
+            {'pk': 'moon'},
+        ])
